@@ -3,7 +3,8 @@ import os
 import tempfile
 
 from django.test import TestCase, override_settings
-from django.contrib.auth.models import User
+from django.conf import settings
+from django.contrib.auth.models import Group, User
 from django.core.files.uploadedfile import SimpleUploadedFile
 from django.urls import reverse
 from unittest.mock import patch
@@ -14,12 +15,13 @@ from astropy.io import fits
 from astropy.table import Table
 import numpy as np
 
-from tom_observations.tests.utils import FakeFacility
+from tom_observations.tests.utils import FakeRoboticFacility
 from tom_observations.tests.factories import TargetFactory, ObservingRecordFactory
-from tom_dataproducts.models import DataProduct, PHOTOMETRY, SPECTROSCOPY, is_fits_image_file
+from tom_dataproducts.models import DataProduct, is_fits_image_file
 from tom_dataproducts.forms import DataProductUploadForm
-from tom_dataproducts.data_processor import DataProcessor
-from tom_dataproducts.data_serializers import SpectrumSerializer
+from tom_dataproducts.processors.photometry_processor import PhotometryProcessor
+from tom_dataproducts.processors.spectroscopy_processor import SpectroscopyProcessor
+from tom_dataproducts.processors.data_serializers import SpectrumSerializer
 from tom_dataproducts.exceptions import InvalidFileFormatException
 from tom_dataproducts.utils import create_image_dataproduct
 from guardian.shortcuts import assign_perm
@@ -37,14 +39,15 @@ def mock_is_fits_image_file(filename):
     return True
 
 
-@override_settings(TOM_FACILITY_CLASSES=['tom_observations.tests.utils.FakeFacility'])
+@override_settings(TOM_FACILITY_CLASSES=['tom_observations.tests.utils.FakeRoboticFacility'],
+                   TARGET_PERMISSIONS_ONLY=True)
 @patch('tom_dataproducts.models.DataProduct.get_preview', return_value='/no-image.jpg')
-class TestObservationDataViews(TestCase):
+class Views(TestCase):
     def setUp(self):
         self.target = TargetFactory.create()
         self.observation_record = ObservingRecordFactory.create(
             target_id=self.target.id,
-            facility=FakeFacility.name,
+            facility=FakeRoboticFacility.name,
             parameters='{}'
         )
         self.data_product = DataProduct.objects.create(
@@ -71,10 +74,10 @@ class TestObservationDataViews(TestCase):
 
     def test_save_dataproduct(self, dp_mock):
         mock_return = [DataProduct(product_id='testdpid', data=SimpleUploadedFile('afile.fits', b'afile'))]
-        with patch.object(FakeFacility, 'save_data_products', return_value=mock_return) as mock:
+        with patch.object(FakeRoboticFacility, 'save_data_products', return_value=mock_return) as mock:
             response = self.client.post(
                 reverse('dataproducts:save', kwargs={'pk': self.observation_record.id}),
-                data={'facility': 'FakeFacility', 'products': ['testdpid']},
+                data={'facility': 'FakeRoboticFacility', 'products': ['testdpid']},
                 follow=True
             )
             self.assertTrue(mock.called)
@@ -110,7 +113,7 @@ class TestObservationDataViews(TestCase):
                 hdul = fits.HDUList([img])
                 hdul.writeto(img_file)
                 self.data_product.data = img_file
-                self.assertTrue(is_fits_image_file(self.data_product.data))
+                self.assertTrue(is_fits_image_file(self.data_product.data.file))
 
     # Table + image data
     def test_is_fits_image_file_table_img(self, dp_mock):
@@ -130,28 +133,29 @@ class TestObservationDataViews(TestCase):
                 hdul = fits.HDUList([img, table])
                 hdul.writeto(tabimg_file)
                 self.data_product.data = tabimg_file
-                self.assertTrue(is_fits_image_file(self.data_product.data))
+                self.assertTrue(is_fits_image_file(self.data_product.data.file))
 
     @patch('tom_dataproducts.models.fits_to_jpg', mock_fits2image)
     @patch('tom_dataproducts.models.find_fits_img_size', mock_find_fits_img_size)
     @patch('tom_dataproducts.models.is_fits_image_file', mock_is_fits_image_file)
     def test_create_jpeg(self, dp_mock):
-        products = DataProduct.objects.filter(tag='image_file')
+        products = DataProduct.objects.filter(data_product_type='image_file')
         self.assertEqual(products.count(), 0)
         resp = create_image_dataproduct(self.data_product)
         self.assertTrue(resp)
-        products = DataProduct.objects.filter(tag='image_file')
+        products = DataProduct.objects.filter(data_product_type='image_file')
         self.assertEqual(products.count(), 1)
 
 
-@override_settings(TOM_FACILITY_CLASSES=['tom_observations.tests.utils.FakeFacility'])
-@patch('tom_dataproducts.views.run_hook')
-class TestUploadDataProducts(TestCase):
+@override_settings(TOM_FACILITY_CLASSES=['tom_observations.tests.utils.FakeRoboticFacility'],
+                   TARGET_PERMISSIONS_ONLY=False)
+@patch('tom_dataproducts.models.DataProduct.get_preview', return_value='/no-image.jpg')
+class TestViewsWithPermissions(TestCase):
     def setUp(self):
         self.target = TargetFactory.create()
         self.observation_record = ObservingRecordFactory.create(
             target_id=self.target.id,
-            facility=FakeFacility.name,
+            facility=FakeRoboticFacility.name,
             parameters='{}'
         )
         self.data_product = DataProduct.objects.create(
@@ -160,34 +164,102 @@ class TestUploadDataProducts(TestCase):
             observation_record=self.observation_record,
             data=SimpleUploadedFile('afile.fits', b'somedata')
         )
-        user = User.objects.create_user(username='test', email='test@example.com')
-        assign_perm('tom_targets.view_target', user, self.target)
-        self.client.force_login(user)
+        self.user = User.objects.create_user(username='aaronrodgers', email='aaron.rodgers@packers.com')
+        self.user2 = User.objects.create_user(username='timboyle', email='tim.boyle@packers.com')
+        assign_perm('tom_targets.view_target', self.user, self.target)
+        assign_perm('tom_targets.view_target', self.user2, self.target)
+        assign_perm('tom_targets.view_dataproduct', self.user, self.data_product)
+        self.client.force_login(self.user)
 
-    def test_upload_data_for_target(self, run_hook_mock):
+    def test_dataproduct_list_on_target(self, dp_mock):
+        response = self.client.get(reverse('tom_targets:detail', kwargs={'pk': self.target.id}))
+        self.assertContains(response, 'afile.fits')
+        self.client.force_login(self.user2)
+
+    def test_dataproduct_list_on_target_unauthorized(self, dp_mock):
+        self.client.force_login(self.user2)
+        response = self.client.get(reverse('tom_targets:detail', kwargs={'pk': self.target.id}))
+        self.assertNotContains(response, 'afile.fits')
+
+    def test_dataproduct_list(self, dp_mock):
+        response = self.client.get(reverse('tom_dataproducts:list'))
+        self.assertContains(response, 'afile.fits')
+
+    def test_dataproduct_list_unauthorized(self, dp_mock):
+        self.client.force_login(self.user2)
+        response = self.client.get(reverse('tom_dataproducts:list'))
+        self.assertNotContains(response, 'afile.fits')
+
+    @override_settings(TOM_FACILITY_CLASSES=['tom_observations.tests.utils.FakeRoboticFacility'],
+                       TARGET_PERMISSIONS_ONLY=False)
+    def test_upload_data_extended_permissions(self, dp_mock):
+        group = Group.objects.create(name='permitted')
+        group.user_set.add(self.user)
         response = self.client.post(
             reverse('dataproducts:upload'),
             {
                 'facility': 'LCO',
                 'files': SimpleUploadedFile('afile.fits', b'afile'),
                 'target': self.target.id,
-                'tag': SPECTROSCOPY[0],
+                'groups': Group.objects.filter(name='permitted'),
+                'data_product_type': settings.DATA_PRODUCT_TYPES['spectroscopy'][0],
                 'observation_timestamp_0': date(2019, 6, 1),
                 'observation_timestamp_1': time(12, 0, 0),
                 'referrer': reverse('targets:detail', kwargs={'pk': self.target.id})
             },
             follow=True
         )
-        self.assertContains(response, 'Successfully uploaded: {0}/none/afile.fits'.format(self.target.identifier))
+        self.assertContains(response, 'afile.fits')
+        self.client.force_login(self.user2)
+        response = self.client.get(reverse('targets:detail', kwargs={'pk': self.target.id}))
+        self.assertNotContains(response, 'afile.fits')
 
-    def test_upload_data_for_observation(self, run_hook_mock):
+
+@override_settings(TOM_FACILITY_CLASSES=['tom_observations.tests.utils.FakeRoboticFacility'],
+                   TARGET_PERMISSIONS_ONLY=True)
+@patch('tom_dataproducts.views.run_data_processor')
+class TestUploadDataProducts(TestCase):
+    def setUp(self):
+        self.target = TargetFactory.create()
+        self.observation_record = ObservingRecordFactory.create(
+            target_id=self.target.id,
+            facility=FakeRoboticFacility.name,
+            parameters='{}'
+        )
+        self.data_product = DataProduct.objects.create(
+            product_id='testproductid',
+            target=self.target,
+            observation_record=self.observation_record,
+            data=SimpleUploadedFile('afile.fits', b'somedata')
+        )
+        self.user = User.objects.create_user(username='test', email='test@example.com')
+        assign_perm('tom_targets.view_target', self.user, self.target)
+        self.client.force_login(self.user)
+
+    def test_upload_data_for_target(self, run_data_processor_mock):
+        response = self.client.post(
+            reverse('dataproducts:upload'),
+            {
+                'facility': 'LCO',
+                'files': SimpleUploadedFile('afile.fits', b'afile'),
+                'target': self.target.id,
+                'data_product_type': settings.DATA_PRODUCT_TYPES['spectroscopy'][0],
+                'observation_timestamp_0': date(2019, 6, 1),
+                'observation_timestamp_1': time(12, 0, 0),
+                'referrer': reverse('targets:detail', kwargs={'pk': self.target.id})
+            },
+            follow=True
+        )
+        self.assertContains(response, 'Successfully uploaded: {0}/none/afile.fits'.format(self.target.name))
+
+    def test_upload_data_for_observation(self, run_data_processor_mock):
         response = self.client.post(
             reverse('dataproducts:upload'),
             {
                 'facility': 'LCO',
                 'files': SimpleUploadedFile('bfile.fits', b'afile'),
                 'observation_record': self.observation_record.id,
-                'tag': SPECTROSCOPY[0],
+                'data_product_type': settings.DATA_PRODUCT_TYPES['spectroscopy'][0],
                 'observation_timestamp_0': date(2019, 6, 1),
                 'observation_timestamp_1': time(12, 0, 0),
                 'referrer': reverse('targets:detail', kwargs={'pk': self.target.id})
@@ -195,21 +267,57 @@ class TestUploadDataProducts(TestCase):
             follow=True
         )
         self.assertContains(response, 'Successfully uploaded: {0}/{1}/bfile.fits'.format(
-            self.target.identifier, FakeFacility.name)
+            self.target.name, FakeRoboticFacility.name)
         )
 
 
+class TestDeleteDataProducts(TestCase):
+    def setUp(self):
+        self.target = TargetFactory.create()
+        self.data_product = DataProduct.objects.create(
+            product_id='testproductid',
+            target=self.target,
+            data=SimpleUploadedFile('afile.fits', b'somedata')
+        )
+        self.user = User.objects.create_user(username='aaronrodgers', email='aaron.rodgers@packers.com')
+        self.user2 = User.objects.create_user(username='timboyle', email='tim.boyle@packers.com')
+        assign_perm('tom_targets.view_target', self.user, self.target)
+        assign_perm('tom_targets.view_target', self.user2, self.target)
+        assign_perm('tom_targets.view_dataproduct', self.user, self.data_product)
+        assign_perm('tom_targets.view_dataproduct', self.user2, self.data_product)
+        assign_perm('tom_targets.delete_dataproduct', self.user, self.data_product)
+        self.client.force_login(self.user)
+
+    def test_delete_data_product_target_permissions_only(self):
+        response = self.client.post(reverse('dataproducts:delete', kwargs={'pk': self.data_product.id}), follow=True)
+        self.assertRedirects(response, reverse('home'))
+        self.assertFalse(DataProduct.objects.filter(product_id='testproductid').exists())
+
+    @override_settings(TARGET_PERMISSIONS_ONLY=False)
+    def test_delete_data_product_unauthorized(self):
+        self.client.force_login(self.user2)
+        response = self.client.post(reverse('dataproducts:delete', kwargs={'pk': self.data_product.id}), follow=True)
+        self.assertRedirects(response, reverse('login') + f'?next=/dataproducts/data/{self.data_product.id}/delete/')
+        self.assertTrue(DataProduct.objects.filter(product_id='testproductid').exists())
+
+    @override_settings(TARGET_PERMISSIONS_ONLY=False)
+    def test_delete_data_product_authorized(self):
+        response = self.client.post(reverse('dataproducts:delete', kwargs={'pk': self.data_product.id}), follow=True)
+        self.assertRedirects(response, reverse('home'))
+        self.assertFalse(DataProduct.objects.filter(product_id='testproductid').exists())
+
+
 class TestDataUploadForms(TestCase):
     def setUp(self):
         self.target = TargetFactory.create()
         self.observation_record = ObservingRecordFactory.create(
             target_id=self.target.id,
-            facility=FakeFacility.name,
+            facility=FakeRoboticFacility.name,
             parameters='{}'
         )
         self.spectroscopy_form_data = {
             'target': self.target.id,
-            'tag': SPECTROSCOPY[0],
+            'data_product_type': settings.DATA_PRODUCT_TYPES['spectroscopy'][0],
             'facility': 'LCO',
             'observation_timestamp_0': date(2019, 6, 1),
             'observation_timestamp_1': time(12, 0, 0),
@@ -217,7 +325,7 @@ class TestDataUploadForms(TestCase):
         }
         self.photometry_form_data = {
             'target': self.target.id,
-            'tag': PHOTOMETRY[0],
+            'data_product_type': settings.DATA_PRODUCT_TYPES['photometry'][0],
             'referrer': 'referrer'
         }
         self.file_data = {
@@ -272,37 +380,42 @@ class TestDataSerializer(TestCase):
             self.serializer.deserialize(json.dumps({'invalid_key': 'value'}))
 
 
-@override_settings(TOM_FACILITY_CLASSES=['tom_observations.tests.utils.FakeFacility'])
+@override_settings(TOM_FACILITY_CLASSES=['tom_observations.tests.utils.FakeRoboticFacility'])
 class TestDataProcessor(TestCase):
     def setUp(self):
         self.target = TargetFactory.create()
         self.data_product = DataProduct.objects.create(
             target=self.target
         )
-        self.data_processor = DataProcessor()
+        self.spectrum_data_processor = SpectroscopyProcessor()
+        self.photometry_data_processor = PhotometryProcessor()
         self.test_file = SimpleUploadedFile('afile.fits', b'somedata')
 
-    @patch('tom_dataproducts.data_processor.DataProcessor._process_spectrum_from_fits')
-    def test_process_spectroscopy_with_fits_file(self, process_data_mock):
+    @patch('tom_dataproducts.processors.spectroscopy_processor.SpectroscopyProcessor._process_spectrum_from_fits',
+           return_value=('', ''))
+    @patch('tom_dataproducts.processors.spectroscopy_processor.SpectrumSerializer.serialize', return_value={})
+    def test_process_spectroscopy_with_fits_file(self, serializer_mock, process_data_mock):
         self.data_product.data.save('spectrum.fits', self.test_file)
-        self.data_processor.process_spectroscopy(self.data_product)
+        self.spectrum_data_processor.process_data(self.data_product)
         process_data_mock.assert_called_with(self.data_product)
 
-    @patch('tom_dataproducts.data_processor.DataProcessor._process_spectrum_from_plaintext')
-    def test_process_spectroscopy_with_plaintext_file(self, process_data_mock):
+    @patch('tom_dataproducts.processors.spectroscopy_processor.SpectroscopyProcessor._process_spectrum_from_plaintext',
+           return_value=('', ''))
+    @patch('tom_dataproducts.processors.spectroscopy_processor.SpectrumSerializer.serialize', return_value={})
+    def test_process_spectroscopy_with_plaintext_file(self, serializer_mock, process_data_mock):
         self.data_product.data.save('spectrum.csv', self.test_file)
-        self.data_processor.process_spectroscopy(self.data_product)
+        self.spectrum_data_processor.process_data(self.data_product)
         process_data_mock.assert_called_with(self.data_product)
 
     def test_process_spectroscopy_with_invalid_file_type(self):
         self.data_product.data.save('spectrum.png', self.test_file)
         with self.assertRaises(InvalidFileFormatException):
-            self.data_processor.process_spectroscopy(self.data_product)
+            self.spectrum_data_processor.process_data(self.data_product)
 
     def test_process_spectrum_from_fits(self):
         with open('tom_dataproducts/tests/test_data/test_spectrum.fits', 'rb') as spectrum_file:
             self.data_product.data.save('spectrum.fits', spectrum_file)
-            spectrum, date_obs = self.data_processor._process_spectrum_from_fits(self.data_product)
+            spectrum, date_obs = self.spectrum_data_processor._process_spectrum_from_fits(self.data_product)
             self.assertTrue(type(spectrum) is Spectrum1D)
             self.assertAlmostEqual(spectrum.flux.mean().value, 2.295068e-14, places=19)
             self.assertAlmostEqual(spectrum.wavelength.mean().value, 6600.478789, places=5)
@@ -310,25 +423,25 @@ class TestDataProcessor(TestCase):
     def test_process_spectrum_from_plaintext(self):
         with open('tom_dataproducts/tests/test_data/test_spectrum.csv', 'rb') as spectrum_file:
             self.data_product.data.save('spectrum.csv', spectrum_file)
-            spectrum, date_obs = self.data_processor._process_spectrum_from_plaintext(self.data_product)
+            spectrum, date_obs = self.spectrum_data_processor._process_spectrum_from_plaintext(self.data_product)
             self.assertTrue(type(spectrum) is Spectrum1D)
             self.assertAlmostEqual(spectrum.flux.mean().value, 1.166619e-14, places=19)
             self.assertAlmostEqual(spectrum.wavelength.mean().value, 3250.744489, places=5)
 
-    @patch('tom_dataproducts.data_processor.DataProcessor._process_photometry_from_plaintext')
+    @patch('tom_dataproducts.processors.photometry_processor.PhotometryProcessor._process_photometry_from_plaintext')
     def test_process_photometry_with_plaintext_file(self, process_data_mock):
         self.data_product.data.save('lightcurve.csv', self.test_file)
-        self.data_processor.process_photometry(self.data_product)
+        self.photometry_data_processor.process_data(self.data_product)
         process_data_mock.assert_called_with(self.data_product)
 
     def test_process_photometry_with_invalid_file_type(self):
         self.data_product.data.save('lightcurve.blah', self.test_file)
         with self.assertRaises(InvalidFileFormatException):
-            self.data_processor.process_photometry(self.data_product)
+            self.photometry_data_processor.process_data(self.data_product)
 
     def test_process_photometry_from_plaintext(self):
         with open('tom_dataproducts/tests/test_data/test_lightcurve.csv', 'rb') as lightcurve_file:
             self.data_product.data.save('lightcurve.csv', lightcurve_file)
-            lightcurve = self.data_processor._process_photometry_from_plaintext(self.data_product)
-            self.assertTrue(type(lightcurve) is dict)
-            self.assertEqual(len(lightcurve), 2)
+            lightcurve = self.photometry_data_processor._process_photometry_from_plaintext(self.data_product)
+            self.assertTrue(type(lightcurve) is list)
+            self.assertEqual(len(lightcurve), 3)
