@@ -5,9 +5,8 @@ from dateutil.parser import parse
 from crispy_forms.layout import Layout, Div, HTML
 from astropy import units as u
 
-from tom_observations.facility import GenericObservationForm
+from tom_observations.facility import BaseRoboticObservationFacility, BaseRoboticObservationForm
 from tom_common.exceptions import ImproperCredentialsException
-from tom_observations.facility import GenericObservationFacility
 from tom_targets.models import Target
 
 try:
@@ -117,7 +116,7 @@ def get_site(progid, location=False):
     return site
 
 
-class GEMObservationForm(GenericObservationForm):
+class GEMObservationForm(BaseRoboticObservationForm):
     """
     The GEMObservationForm defines and collects the parameters for the Gemini
     Target of Opportunity (ToO) observation request API. The Gemini ToO process is described at
@@ -254,10 +253,8 @@ class GEMObservationForm(GenericObservationForm):
                                    label='UT Timing Window Start [Date Time]')
     window_duration = forms.IntegerField(required=False, min_value=1, label='Timing Window Duration [hr]')
 
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.helper.layout = Layout(
-            self.common_layout,
+    def layout(self):
+        return Div(
             HTML('<big>Observation Parameters</big>'),
             Div(
                 Div(
@@ -412,7 +409,12 @@ class GEMObservationForm(GenericObservationForm):
         return payloads
 
 
-class GEMFacility(GenericObservationFacility):
+class GEMFacility(BaseRoboticObservationFacility):
+    """
+    The ``GEMFacility`` is the interface to the Gemini Telescope. For information regarding Gemini observing and the
+    available parameters, please see https://www.gemini.edu/sciops/observing-gemini.
+    """
+
     name = 'GEM'
     observation_types = [('OBSERVATION', 'Gemini Observation')]
 
@@ -461,6 +463,9 @@ class GEMFacility(GenericObservationFacility):
         # return PORTAL_URL + '/requests/' + observation_id
         return ''
 
+    def get_start_end_keywords(self):
+        return ('window_start', 'window_end')
+
     @classmethod
     def get_terminal_observing_states(clz):
         return TERMINAL_OBSERVING_STATES
