@@ -1,4 +1,5 @@
-from django.db import models
+from django.db import models, transaction
+from django.core.exceptions import ValidationError
 from django.urls import reverse
 from django.forms.models import model_to_dict
 from django.conf import settings
@@ -7,41 +8,38 @@ from datetime import datetime
 
 from tom_common.hooks import run_hook
 
-
-GLOBAL_TARGET_FIELDS = ['identifier', 'name', 'name2', 'name3', 'type']
+GLOBAL_TARGET_FIELDS = ['name', 'type']
 
 SIDEREAL_FIELDS = GLOBAL_TARGET_FIELDS + [
-    'ra', 'dec', 'epoch', 'pm_ra', 'pm_dec',
-    'galactic_lng', 'galactic_lat', 'distance', 'distance_err'
+    'ra', 'dec', 'epoch', 'pm_ra', 'pm_dec', 'galactic_lng', 'galactic_lat', 'distance', 'distance_err'
 ]
 
 NON_SIDEREAL_FIELDS = GLOBAL_TARGET_FIELDS + [
-    'scheme', 'mean_anomaly', 'arg_of_perihelion',
-    'lng_asc_node', 'inclination', 'mean_daily_motion', 'semimajor_axis',
-    'eccentricity', 'epoch', 'epoch_of_perihelion', 'ephemeris_period',
-    'ephemeris_period_err', 'ephemeris_epoch', 'ephemeris_epoch_err'
+    'scheme', 'mean_anomaly', 'arg_of_perihelion', 'lng_asc_node', 'inclination', 'mean_daily_motion', 'semimajor_axis',
+    'eccentricity', 'epoch_of_elements', 'epoch_of_perihelion', 'ephemeris_period', 'ephemeris_period_err',
+    'ephemeris_epoch', 'ephemeris_epoch_err', 'perihdist'
 ]
 
 REQUIRED_SIDEREAL_FIELDS = ['ra', 'dec']
-REQUIRED_NON_SIDEREAL_FIELDS = NON_SIDEREAL_FIELDS
+REQUIRED_NON_SIDEREAL_FIELDS = [
+    'scheme', 'epoch_of_elements', 'inclination', 'lng_asc_node', 'arg_of_perihelion', 'eccentricity',
+]
+# Additional non-sidereal fields that are required for specific orbital element
+# schemes
+REQUIRED_NON_SIDEREAL_FIELDS_PER_SCHEME = {
+    'MPC_COMET': ['perihdist', 'epoch_of_perihelion'],
+    'MPC_MINOR_PLANET': ['mean_anomaly', 'semimajor_axis'],
+    'JPL_MAJOR_PLANET': ['mean_daily_motion', 'mean_anomaly', 'semimajor_axis']
+}
 
 
 class Target(models.Model):
     """
     Class representing a target in a TOM
 
-    :param identifier: The identifier for this object, e.g. Kelt-16b.
-    :type identifier: str
-
     :param name: The name of this target e.g. Barnard\'s star.
     :type name: str
 
-    :param name2: An alternative name for this target.
-    :type name2: str
-
-    :param name3: An alternative name for this target.
-    :type name3: str
-
     :param type: The type of this target.
     :type type: str
 
@@ -84,6 +82,9 @@ class Target(models.Model):
     :param scheme: Orbital Element Scheme
     :type scheme: str
 
+    :param epoch_of_elements: Epoch of elements in JD.
+    :type epoch_of_elements: float
+
     :param mean_anomaly: Angle in degrees.
     :type mean_anomaly: float
 
@@ -131,19 +132,9 @@ class Target(models.Model):
         ('JPL_MAJOR_PLANET', 'JPL Major Planet')
     )
 
-    identifier = models.CharField(
-        max_length=100, verbose_name='Identifier', help_text='The identifier for this object, e.g. Kelt-16b.'
-    )
     name = models.CharField(
-        max_length=100, default='', verbose_name='Name', help_text='The name of this target e.g. Barnard\'s star.'
-    )
-    name2 = models.CharField(
-        max_length=100, default='', verbose_name='Name 2', help_text='An alternative name for this target',
-        blank=True
-    )
-    name3 = models.CharField(
-        max_length=100, default='', verbose_name='Name 3', help_text='An alternative name for this target',
-        blank=True
+        max_length=100, default='', verbose_name='Name', help_text='The name of this target e.g. Barnard\'s star.',
+        unique=True
     )
     type = models.CharField(
         max_length=100, choices=TARGET_TYPES, verbose_name='Target Type', help_text='The type of this target.'
@@ -163,7 +154,7 @@ class Target(models.Model):
         null=True, blank=True, verbose_name='Declination', help_text='Declination, in degrees.'
     )
     epoch = models.FloatField(
-        null=True, blank=True, verbose_name='Epoch of Elements', help_text='Julian Years. Max 2100.'
+        null=True, blank=True, verbose_name='Epoch', help_text='Julian Years. Max 2100.'
     )
     parallax = models.FloatField(
         null=True, blank=True, verbose_name='Parallax', help_text='Parallax, in milliarcseconds.'
@@ -190,6 +181,9 @@ class Target(models.Model):
     scheme = models.CharField(
         max_length=50, choices=TARGET_SCHEMES, verbose_name='Orbital Element Scheme', default='', blank=True
     )
+    epoch_of_elements = models.FloatField(
+        null=True, blank=True, verbose_name='Epoch of Elements', help_text='Julian date.'
+    )
     mean_anomaly = models.FloatField(
         null=True, blank=True, verbose_name='Mean Anomaly', help_text='Angle in degrees.'
     )
@@ -229,10 +223,14 @@ class Target(models.Model):
     ephemeris_epoch_err = models.FloatField(
         null=True, blank=True, verbose_name='Ephemeris Epoch Error', help_text='Days'
     )
+    perihdist = models.FloatField(
+        null=True, blank=True, verbose_name='Perihelion Distance', help_text='AU'
+    )
 
     class Meta:
         ordering = ('id',)
 
+    @transaction.atomic
     def save(self, *args, **kwargs):
         """
         Saves Target model data to the database, including extra fields. After saving to the database, also runs the
@@ -242,16 +240,39 @@ class Target(models.Model):
             * extras (`dict`): dictionary of key/value pairs representing target attributes
         """
         extras = kwargs.pop('extras', {})
+        names = kwargs.pop('names', [])
+
         created = False if self.id else True
         super().save(*args, **kwargs)
+
+        if created:
+            for extra_field in settings.EXTRA_FIELDS:
+                if extra_field.get('default') is not None:
+                    TargetExtra(target=self, key=extra_field['name'], value=extra_field.get('default')).save()
+
         for k, v in extras.items():
             target_extra, _ = TargetExtra.objects.get_or_create(target=self, key=k)
             target_extra.value = v
             target_extra.save()
-        run_hook('target_post_save', target=self, created=created)
+
+        for name in names:
+            name, _ = TargetName.objects.get_or_create(target=self, name=name)
+            name.save()
+
+        if not created:
+            run_hook('target_post_save', target=self, created=created)
+
+    def validate_unique(self, *args, **kwargs):
+        """
+        Ensures that Target.name and all aliases of the target are unique. Called automatically on save.
+        """
+        super().validate_unique(*args, **kwargs)
+        for alias in self.aliases.all():
+            if alias.name == self.name:
+                raise ValidationError('Target name and target aliases must be unique')
 
     def __str__(self):
-        return str(self.identifier)
+        return str(self.name)
 
     def get_absolute_url(self):
         return reverse('targets:detail', kwargs={'pk': self.id})
@@ -261,10 +282,20 @@ class Target(models.Model):
         Gets the ``DataProduct`` associated with this ``Target`` that is a FITS file and is uniquely marked as
         "featured".
 
-        :returns: ``DataProduct`` with tag of ``fits_file`` and featured as ``True``
+        :returns: ``DataProduct`` with data_product_type of ``fits_file`` and featured as ``True``
         :rtype: DataProduct
         """
-        return self.dataproduct_set.filter(tag='fits_file', featured=True).first()
+        return self.dataproduct_set.filter(data_product_type='fits_file', featured=True).first()
+
+    @property
+    def names(self):
+        """
+        Gets a list with the name and aliases of this target
+
+        :returns: list of all names and `TargetName` values associated with this target
+        :rtype: list
+        """
+        return [self.name] + [alias.name for alias in self.aliases.all()]
 
     @property
     def future_observations(self):
@@ -322,6 +353,43 @@ class Target(models.Model):
         return model_to_dict(self, fields=fields_for_type)
 
 
+class TargetName(models.Model):
+    """
+    Class representing an alternative name for a ``Target``.
+
+    :param target: The ``Target`` object this ``TargetName`` is associated with.
+
+    :param name: The name that this ``TargetName`` object represents.
+    :type name: str
+
+    :param created: The time at which this target name was created in the TOM database.
+    :type created: datetime
+
+    :param modified: The time at which this target name was modified in the TOM database.
+    :type modified: datetime
+    """
+    target = models.ForeignKey(Target, on_delete=models.CASCADE, related_name='aliases')
+    name = models.CharField(max_length=100, unique=True, verbose_name='Alias for target')
+    created = models.DateTimeField(
+        auto_now_add=True, help_text='The time which this target name was created.'
+    )
+    modified = models.DateTimeField(
+        auto_now=True, verbose_name='Last Modified',
+        help_text='The time which this target name was changed in the TOM database.'
+    )
+
+    def __str__(self):
+        return self.name
+
+    def validate_unique(self, *args, **kwargs):
+        """
+        Ensures that Target.name and all aliases of the target are unique. Called automatically on save.
+        """
+        super().validate_unique(*args, **kwargs)
+        if self.name == self.target.name:
+            raise ValidationError('Target name and target aliases must be unique')
+
+
 class TargetExtra(models.Model):
     """
     Class representing a list of targets in a TOM.
@@ -345,7 +413,7 @@ class TargetExtra(models.Model):
     """
     target = models.ForeignKey(Target, on_delete=models.CASCADE)
     key = models.CharField(max_length=200)
-    value = models.TextField()
+    value = models.TextField(blank=True, default='')
     float_value = models.FloatField(null=True, blank=True)
     bool_value = models.BooleanField(null=True, blank=True)
     time_value = models.DateTimeField(null=True, blank=True)
@@ -409,14 +477,21 @@ class TargetList(models.Model):
 
     :param targets: Set of ``Target`` objects associated with this ``TargetList``
 
-    :param created: The time at which this object was created.
+    :param created: The time at which this target list was created.
     :type created: datetime
+
+    :param modified: The time at which this target list was modified in the TOM database.
+    :type modified: datetime
     """
     name = models.CharField(max_length=200, help_text='The name of the target list.')
     targets = models.ManyToManyField(Target)
     created = models.DateTimeField(
         auto_now_add=True, help_text='The time which this target list was created in the TOM database.'
     )
+    modified = models.DateTimeField(
+        auto_now=True, verbose_name='Last Modified',
+        help_text='The time which this target list was changed in the TOM database.'
+    )
 
     class Meta:
         ordering = ('-created', 'name',)
