@@ -13,9 +13,9 @@ urlpatterns = [
     path('import/', TargetImportView.as_view(), name='import'),
     path('export/', TargetExportView.as_view(), name='export'),
     path('add-remove-grouping/', TargetAddRemoveGroupingView.as_view(), name='add-remove-grouping'),
-    path('<pk>/update/', TargetUpdateView.as_view(), name='update'),
-    path('<pk>/delete/', TargetDeleteView.as_view(), name='delete'),
-    path('<pk>/', TargetDetailView.as_view(), name='detail'),
+    path('<int:pk>/update/', TargetUpdateView.as_view(), name='update'),
+    path('<int:pk>/delete/', TargetDeleteView.as_view(), name='delete'),
+    path('<int:pk>/', TargetDetailView.as_view(), name='detail'),
     path('targetgrouping/<int:pk>/delete/', TargetGroupingDeleteView.as_view(), name='delete-group'),
     path('targetgrouping/create/', TargetGroupingCreateView.as_view(), name='create-group')
 ]
